VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "TaskbarList"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

' **********************************************************************************************************
' Module:      	TaskbarList
' Author:      	Philipp Stiefel (phil@codekabinett.com)
' Purpose:     	Wrapper around the TaskbarList COM Component from Shell32
' Further info: http://codekabinett.com/rdumps.php?Lang=2&targetDoc=raw-com-api-vba-hide-taskbar-button-window-itaskbarlist
' Usage rights: You are free to use this class module in any of your applications free of charge, 
'				as long as this notice remains unchanged.
' **********************************************************************************************************

Option Compare Database
Option Explicit

Private Const CLASS_NAME As String = "TaskbarList"


Private Const S_OK As Long = 0

Private Const IID_IUnknown As String = "{00000000-0000-0000-C000-000000000046}"
Private Const IID_ITASKBAR_LIST As String = "{56FDF342-FD6D-11D0-958A-006097C9A090}"

Private Const CLSID_TASKBAR_LIST As String = "{56FDF344-FD6D-11D0-958A-006097C9A090}"      ' Task Bar Communication

Private Const CLSCTX_INPROC_SERVER As Long = 1

Private Const CC_STDCALL = 4

Private Declare Function DispCallFunc Lib "oleaut32.dll" (ByVal pvInstance As Long, ByVal offsetVtable As Long, ByVal CallConv As Long, ByVal vartypeReturn As Integer, ByVal paramCount As Long, ByRef paramTypes As Integer, ByRef paramValues As Long, ByRef returnValue As Variant) As Long

Private Declare Function CLSIDFromString Lib "OLE32.DLL" (ByVal lpsz As Long, ByRef pClsid As GUID) As Long
Private Declare Function IIDFromString Lib "OLE32.DLL" (ByVal lpsz As Long, ByRef lpiid As GUID) As Long
Private Declare Function CoCreateInstance Lib "OLE32.DLL" (ByRef rclsid As GUID, ByVal pUnkOuter As Long, ByVal dwClsContext As Long, ByRef riid As GUID, ByRef ppv As Object) As Long

Private Type GUID
  Data1 As Long
  Data2 As Integer
  Data3 As Integer
  Data4(0 To 7) As Byte
End Type

Private Enum ITaskbarListVtable
    OfHrInit = 12
    OfAddTab = 16
    OfDeleteTab = 20
    OfActivateTab = 24
    OfSetActivateAlt = 28
End Enum

Private Enum IUnknownVtable
    OfQueryInterface = 0
    OfAddRef = 4
    OfReleaseRef = 8
End Enum

Private m_pITaskbarList As Long


Private Function getIUnknownOfTaskbar() As Object
    
    Dim retVal As Object    ' IUnknown
    ' WARNING: retVal is NOT AN OBJECT, it is an instance of IUnknown. Inspecting the variable in Debug Mode will crash your VBA Application
    
    Dim apiRetVal As Long
  
    Dim IIDIUnknownGuid As GUID
    IIDFromString StrPtr(IID_IUnknown), IIDIUnknownGuid
    
    Dim CLSID As GUID
    CLSIDFromString StrPtr(CLSID_TASKBAR_LIST), CLSID

    apiRetVal = CoCreateInstance(CLSID, 0, CLSCTX_INPROC_SERVER, IIDIUnknownGuid, retVal)
    If apiRetVal = S_OK Then
        Set getIUnknownOfTaskbar = retVal
    Else
        Err.Raise vbObjectError Or 1, CLASS_NAME, "Failed to create instance of Taskbar/IUnknown."
    End If
    
End Function


Private Function getPointerToITaskbarList(ByRef IUnknownOfTaskbar As Object) As Long

    Dim hResult As Long
    Dim retVal As Long

    Dim varTypes(1) As Integer
    Dim ptrVarValues(1) As Long
    Dim apiRetVal As Variant
    
    
    Dim IIDITaskbarListGuid As GUID
    IIDFromString StrPtr(IID_ITASKBAR_LIST), IIDITaskbarListGuid

    Dim valueWrapper0 As Variant
    Dim valueWrapper1 As Variant

    valueWrapper0 = VarPtr(IIDITaskbarListGuid)
    valueWrapper1 = VarPtr(retVal)

    ptrVarValues(0) = VarPtr(valueWrapper0)
    ptrVarValues(1) = VarPtr(valueWrapper1)
    
    varTypes(0) = VbVarType.vbLong
    varTypes(1) = VarType(retVal)
    
    Dim paramCount As Long
    paramCount = 2
    
    Dim objAdr As Long
    objAdr = ObjPtr(IUnknownOfTaskbar)
    
    hResult = DispCallFunc(objAdr, IUnknownVtable.OfQueryInterface, CC_STDCALL, VbVarType.vbLong, paramCount, varTypes(0), ptrVarValues(0), apiRetVal)

    If hResult = S_OK Then
        hResult = apiRetVal
        
        If hResult = S_OK Then
            getPointerToITaskbarList = retVal
        Else
            Err.Raise vbObjectError Or 2, CLASS_NAME, "Failed to get ITASKBAR_LIST interface pointer. IUnknown.QueryInterface HRESULT: 0x" & Hex(hResult)
        End If
    Else
        Err.Raise vbObjectError Or 2, CLASS_NAME, "Failed to get ITASKBAR_LIST interface pointer. DispCallFunc HRESULT: 0x" & Hex(hResult)
    End If
        
End Function

Private Sub Class_Initialize()
    m_pITaskbarList = getPointerToITaskbarList(getIUnknownOfTaskbar())
    
    If m_pITaskbarList <> 0 Then
        HrInit
    End If
    
End Sub

Private Sub HrInit()

    Dim hResult As Long
    Dim apiRetVal As Variant
    
    hResult = DispCallFunc(m_pITaskbarList, ITaskbarListVtable.OfHrInit, CC_STDCALL, VbVarType.vbLong, 0, 0, 0, apiRetVal)

    If hResult = S_OK Then
        hResult = apiRetVal
        
        If hResult <> S_OK Then
            Err.Raise vbObjectError Or 3, CLASS_NAME, "HRInit failed. ITaskbarList.HrInit HRESULT: 0x" & Hex(hResult)
        End If
    Else
        Err.Raise vbObjectError Or 3, CLASS_NAME, "HRInit failed. DispCallFunc HRESULT: 0x" & Hex(hResult)
    End If

End Sub

Public Sub AddTab(ByVal hwnd As Long)
    InvokeITaskbarListMember ITaskbarListVtable.OfAddTab, "AddTab", hwnd
End Sub

Public Sub DeleteTab(ByVal hwnd As Long)
    InvokeITaskbarListMember ITaskbarListVtable.OfDeleteTab, "DeleteTab", hwnd
End Sub

Public Sub ActivateTab(ByVal hwnd As Long)
    InvokeITaskbarListMember ITaskbarListVtable.OfActivateTab, "ActivateTab", hwnd
End Sub

Public Sub SetActiveAlt(ByVal hwnd As Long)
    InvokeITaskbarListMember ITaskbarListVtable.OfSetActivateAlt, "SetActiveAlt", hwnd
End Sub

Private Sub InvokeITaskbarListMember(ByVal VTableOffset As Long, ByVal MethodName As String, ByVal hwnd As Long)

    Dim hResult As Long
    Dim varTypes(0 To 0) As Integer
    Dim ptrVarValues(0 To 0) As Long
    Dim apiRetVal As Variant
    Dim valueWrapper0 As Variant
    
    valueWrapper0 = hwnd
    ptrVarValues(0) = VarPtr(valueWrapper0)

    varTypes(0) = VarType(hwnd)

    hResult = DispCallFunc(m_pITaskbarList, VTableOffset, CC_STDCALL, VbVarType.vbLong, 1, varTypes(0), ptrVarValues(0), apiRetVal)

    If hResult = S_OK Then
        hResult = apiRetVal

        If hResult <> S_OK Then
            Err.Raise vbObjectError Or 4, CLASS_NAME, MethodName & " failed. ITaskbarList." & MethodName & " HRESULT: 0x" & Hex(hResult)
        End If
    Else
        Err.Raise vbObjectError Or 4, CLASS_NAME, MethodName & " failed. DispCallFunc HRESULT: 0x" & Hex(hResult)
    End If

End Sub

Private Sub Class_Terminate()
    
    Dim dummyRetVal As Variant

    If m_pITaskbarList <> 0 Then
        DispCallFunc m_pITaskbarList, IUnknownVtable.OfReleaseRef, CC_STDCALL, VbVarType.vbLong, 0, 0, 0, dummyRetVal
    End If

End Sub

' **********************************************************************************************************
' Module:      	TaskbarList
' Author:      	Philipp Stiefel (phil@codekabinett.com)
' Purpose:     	Wrapper around the TaskbarList COM Component from Shell32
' Further info: http://codekabinett.com/rdumps.php?Lang=2&targetDoc=raw-com-api-vba-hide-taskbar-button-window-itaskbarlist
' Usage rights: You are free to use this class module in any of your applications free of charge, 
'				as long as this notice remains unchanged.
' **********************************************************************************************************
